#include <math.h>
#include <sys/time.h>
#include <stdlib.h>
#include <errno.h>
#include "anw.h"
#include "intel.h"

int mycolor;
int lastmove=Kerr,sugmove=Kerr;

const int moves[12][2]={{0,-1}, {1,0}, {0,1}, {-1,0},
{-1,-1}, {1,-1}, {1,1}, {-1,1}, {0,-2}, {2,0}, {0,2}, {-2,0}};
char *pixels;
int WIDTH,HEIGHT;
int x,y;
int odds[4];

typedef unsigned char u8_t;

// pure now, used to contain debugging stuff
inline int
add(int to,int what)
{
	odds[to]+=what;
}

#define INFO_PROTO 0
#define INFO_MYCOLOR 1
#define INFO_MODE 2
# define INFO_MODE_KILLER 2
# define INFO_MODE_WRAP 4
#define INFO_YRES 3
#define INFO_XRES 4
#define INFO_MAX 4

char info[INFO_MAX+2];

void
int_init()
{
	int i;
	i=-1;
	do {
		if (i<=INFO_MAX) i++;
		receive(info+i,1,1);
	} while (info[i]);
	if (info[INFO_PROTO]!=1) {
		errno=EBADE;
		gr_error("protocol");
	}
	mycolor=info[INFO_MYCOLOR]+1;
	WIDTH=info[INFO_XRES];
	HEIGHT=info[INFO_YRES];
	pixels=(char *)malloc(WIDTH*HEIGHT);
	if (!pixels) gr_error("malloc");
	srand(time(NULL));
}

int // FREE:BUSY:HEAD
state(int direction)
{
	int tox,toy;
	direction=direction/4*4+(direction+sugmove)%4;
	tox=x+moves[direction][0];
	toy=y+moves[direction][1];
 	if (tox<0 || toy<1 || tox>=WIDTH || toy>=HEIGHT) return 1;
	if (!pixels[toy*WIDTH+tox] || pixels[toy*WIDTH+tox]=='*') return 0;
	if (pixels[toy*WIDTH+tox]=='@') return 2;
	return 1;
}

void
doheadsjob()
{
}

void
dobotsjob()
{
	int starx,stary,X,Y,dist=1000;
	int gotox,gotoy;
	double angle;
	for(Y=0;Y<HEIGHT;Y++)
		for(X=0;X<WIDTH;X++) {
			if (pixels[Y*WIDTH+X]!='*') continue;
			if (abs(x-X)+abs(y-Y)>dist) continue;
			dist=abs(x-X)+abs(y-Y);
			gotox=X-x; gotoy=Y-y;
		}
	angle=atan2(-gotoy,gotox);
	angle=M_PI/2*(1-sugmove)-angle;
	for(X=0;X<4;X++)
		add(X,(int)(IFAHEAD*cos(M_PI/2*X-angle)));
	doheadsjob();
}

void
doavoidsjob()
{
	int i;
	for(i=0;i<4;i++)
		switch (state(i)) {
		case 2:
			add((i+1)%4,IFHEAD);
			add((i+3)%4,IFHEAD);
			/* fall-through */
		case 1:
			add(i,IFBUSY);
			break;
		}
	for(;i<8;i++)
		switch (state(i)) {
		case 1:
			add(i-4,IFNEAR);
			add((i-1)%4,IFNEAR);
			break;
		case 2:
			add(i-4,IFHEAD);
			add((i-1)%4,IFHEAD);
			break;
		}
	for(;i<12;i++)
		if (state(i)==2) {
			add(i-8,IFHEAD);
			add((i-1)%4,IFHEAD);
		}
}

void
findsugmove()
{
	int i,j=0;
	for(i=1;i<4;i++)
		if (odds[i]<odds[j]) j=i;
	sugmove=(sugmove+j)%4+1;	
}

void
findnextstep(const int ex,const int wy)
{
	int i;
	x=ex;y=wy;
	for(i=0;i<4;i++) {
		odds[i]=0;
		add(i,rand()%(IFRANDOM+1));
	}
	if (!int_bot) {sugmove=Kerr;return;}
	sugmove=lastmove?lastmove-1:0;
	if (int_bot) dobotsjob();
	if (int_bot) doavoidsjob();
	findsugmove();
	if (sugmove==lastmove) sugmove=Kerr;
}

void
int_pixel(struct screen *pix)
{
	if (pix->xpos>=WIDTH || pix->ypos>=HEIGHT) return;
	pixels[pix->ypos*WIDTH+pix->xpos] = pix->chr==' '?0:pix->chr;
	if (pix->chr=='@' && pix->color==mycolor)
		findnextstep(pix->xpos,pix->ypos);
}

void
int_die()
{
	lastmove=sugmove=Kerr;
}
